@@ -3,7 +3,7 @@
 /*
     bqaudioio
 
-    Copyright 2007-2016 Particular Programs Ltd.
+    Copyright 2007-2021 Particular Programs Ltd.
 
     Permission is hereby granted, free of charge, to any person
     obtaining a copy of this software and associated documentation
@@ -90,6 +90,7 @@ PulseAudioIO::PulseAudioIO(Mode mode,
     m_done(false),
     m_captureReady(false),
     m_playbackReady(false),
+    m_aboutToAct(false),
     m_suspended(false)
 {
     log("PulseAudioIO: starting");
@@ -199,6 +200,7 @@ PulseAudioIO::~PulseAudioIO()
     
         {
             if (m_loop) {
+                m_aboutToAct = true;
                 pa_mainloop_wakeup(m_loop);
             }
         
@@ -206,6 +208,7 @@ PulseAudioIO::~PulseAudioIO()
             lock_guard<mutex> lguard(m_loopMutex);
             lock_guard<mutex> sguard(m_streamMutex);
 
+            m_aboutToAct = false;
             m_done = true;
 
             if (m_loop) {
@@ -272,7 +275,14 @@ PulseAudioIO::threadRun()
             }
         }
 
-        this_thread::yield();
+        if (m_aboutToAct) {
+            // another thread awoke us because it wanted to do
+            // something specific - sleep for a moment to ensure it
+            // can acquire its lock
+            this_thread::sleep_for(std::chrono::milliseconds(100));
+        } else {
+            this_thread::yield();
+        }
 
         {
 #ifdef DEBUG_PULSE_AUDIO_IO
@@ -588,7 +598,7 @@ PulseAudioIO::streamStateChanged(pa_stream *stream)
             if (m_target && (stream == m_in)) {
                 m_target->setSystemRecordSampleRate(m_sampleRate);
                 m_target->setSystemRecordChannelCount(m_inSpec.channels);
-                if (pa_stream_get_latency(m_out, &latency, &negative)) {
+                if (pa_stream_get_latency(m_in, &latency, &negative)) {
                     log("streamStateChanged: Failed to query record latency");
                 } else {
                     ostringstream os;
@@ -619,7 +629,10 @@ PulseAudioIO::streamStateChanged(pa_stream *stream)
 void
 PulseAudioIO::suspend()
 {
-    if (m_loop) pa_mainloop_wakeup(m_loop);
+    if (m_loop) {
+        m_aboutToAct = true;
+        pa_mainloop_wakeup(m_loop);
+    }
     
 #ifdef DEBUG_PULSE_AUDIO_IO
     cerr << "PulseAudioIO::suspend: locking all mutexes" << endl;
@@ -639,6 +652,7 @@ PulseAudioIO::suspend()
     cerr << "PulseAudioIO::suspend: stream mutex ok" << endl;
 #endif
 
+    m_aboutToAct = false;
     if (m_done) return;
     
     if (m_in) {
@@ -661,7 +675,10 @@ PulseAudioIO::suspend()
 void
 PulseAudioIO::resume()
 {
-    if (m_loop) pa_mainloop_wakeup(m_loop);
+    if (m_loop) {
+        m_aboutToAct = true;
+        pa_mainloop_wakeup(m_loop);
+    }
     
 #ifdef DEBUG_PULSE_AUDIO_IO
     cerr << "PulseAudioIO::resume: locking all mutexes" << endl;
@@ -680,6 +697,8 @@ PulseAudioIO::resume()
 #ifdef DEBUG_PULSE_AUDIO_IO
     cerr << "PulseAudioIO::suspend: stream mutex ok" << endl;
 #endif
+
+    m_aboutToAct = false;
     if (m_done) return;
 
     if (m_in) {
