@@ -6,7 +6,7 @@
     A small library wrapping various FFT implementations for some
     common audio processing use cases.
 
-    Copyright 2007-2015 Particular Programs Ltd.
+    Copyright 2007-2021 Particular Programs Ltd.
 
     Permission is hereby granted, free of charge, to any person
     obtaining a copy of this software and associated documentation
@@ -77,11 +77,18 @@ BOOST_AUTO_TEST_SUITE(TestFFT)
     for (int cmp_i = 0; cmp_i < (int)(sizeof(a)/sizeof(a[0])); ++cmp_i) { \
         BOOST_CHECK_SMALL(a[cmp_i] - x, epsf); \
     }
-
+#define COMPARE_ARR(a, b, n) \
+    for (int cmp_i = 0; cmp_i < n; ++cmp_i) { \
+        BOOST_CHECK_SMALL(a[cmp_i] - b[cmp_i], eps); \
+    }
 #define COMPARE_SCALED(a, b, s)						\
     for (int cmp_i = 0; cmp_i < (int)(sizeof(a)/sizeof(a[0])); ++cmp_i) { \
         BOOST_CHECK_SMALL(a[cmp_i]/s - b[cmp_i], eps); \
     }
+#define COMPARE_SCALED_N(a, b, n, s) \
+    for (int cmp_i = 0; cmp_i < n; ++cmp_i) { \
+        BOOST_CHECK_SMALL(a[cmp_i]/s - b[cmp_i], eps); \
+    }
 #define COMPARE_SCALED_F(a, b, s)						\
     for (int cmp_i = 0; cmp_i < (int)(sizeof(a)/sizeof(a[0])); ++cmp_i) { \
         BOOST_CHECK_SMALL(a[cmp_i]/s - b[cmp_i], epsf); \
@@ -109,13 +116,13 @@ BOOST_AUTO_TEST_SUITE(TestFFT)
     ONE_IMPL_AUTO_TEST_CASE(name, medialib); \
     ONE_IMPL_AUTO_TEST_CASE(name, openmax); \
     ONE_IMPL_AUTO_TEST_CASE(name, sfft); \
-    ONE_IMPL_AUTO_TEST_CASE(name, cross); \
+    ONE_IMPL_AUTO_TEST_CASE(name, builtin); \
     ONE_IMPL_AUTO_TEST_CASE(name, dft); \
     void performTest_##name ()
 
 std::string all_implementations[] = {
     "ipp", "vdsp", "fftw", "sfft", "openmax",
-    "medialib", "kissfft", "cross", "dft"
+    "medialib", "kissfft", "builtin", "dft"
 };
 
 BOOST_AUTO_TEST_CASE(showImplementations)
@@ -175,6 +182,28 @@ ALL_IMPL_AUTO_TEST_CASE(sine)
     COMPARE_SCALED(back, in, 4);
 }
 
+ALL_IMPL_AUTO_TEST_CASE(sine_8)
+{
+    // Longer sine. With only 4 elements, the real transform only
+    // needs to get the DC and Nyquist bins right for its two complex
+    // sub-transforms. We need a longer test to check the real
+    // transform is working properly.
+    double cospi4 = 0.5 * sqrt(2.0);
+    double in[] = { 0, cospi4, 1.0, cospi4, 0.0, -cospi4, -1.0, -cospi4 };
+    double re[5], im[5];
+    USING_FFT(8);
+    fft.forward(in, re, im);
+    COMPARE_ALL(re, 0.0);
+    COMPARE_ZERO(im[0]);
+    COMPARE(im[1], -4.0);
+    COMPARE_ZERO(im[2]);
+    COMPARE_ZERO(im[3]);
+    COMPARE_ZERO(im[4]);
+    double back[8];
+    fft.inverse(re, im, back);
+    COMPARE_SCALED(back, in, 8);
+}
+
 ALL_IMPL_AUTO_TEST_CASE(cosine)
 {
     // Cosine. Output is purely real
@@ -190,6 +219,25 @@ ALL_IMPL_AUTO_TEST_CASE(cosine)
     fft.inverse(re, im, back);
     COMPARE_SCALED(back, in, 4);
 }
+
+ALL_IMPL_AUTO_TEST_CASE(cosine_8)
+{
+    // Longer cosine.
+    double cospi4 = 0.5 * sqrt(2.0);
+    double in[] = { 1.0, cospi4, 0.0, -cospi4, -1.0, -cospi4, 0.0, cospi4 };
+    double re[5], im[5];
+    USING_FFT(8);
+    fft.forward(in, re, im);
+    COMPARE_ALL(im, 0.0);
+    COMPARE_ZERO(re[0]);
+    COMPARE(re[1], 4.0);
+    COMPARE_ZERO(re[2]);
+    COMPARE_ZERO(re[3]);
+    COMPARE_ZERO(re[4]);
+    double back[8];
+    fft.inverse(re, im, back);
+    COMPARE_SCALED(back, in, 8);
+}
 	
 ALL_IMPL_AUTO_TEST_CASE(sineCosine)
 {
@@ -722,4 +770,76 @@ ALL_IMPL_AUTO_TEST_CASE(sineCosine_7)
 }
 
 
+/*
+ * 6. Slightly longer transforms of pseudorandom data.
+ */
+
+ALL_IMPL_AUTO_TEST_CASE(random_precalc_16)
+{
+    double in[] = {
+        -0.24392125308057722, 0.03443898163344272, 0.3448145656738877,
+        -0.9625837464603908, 3.366568317669671, 0.9947191221586653,
+        -1.5038984435999945, 1.3859898682581235, -1.1230576306688778,
+        -1.6757487116512024, -1.5874436867863229, -2.0794018781307155,
+        -0.5450152775818973, 0.7530907176983748, 1.0743170685904255,
+        3.1787609811018775
+    };
+    double expected_re[] = {
+        1.41162899482, 7.63975551593, -1.20622641052, -1.77829578443,
+        3.12678465246, -2.84220463109, -7.17083743716, 0.497290409945,
+        -1.84690167439,
+    };
+    double expected_im[] = {
+        0.0, -4.67826048083, 8.58829211964, 4.96449646815,
+        1.41626511493, -3.77219223978, 6.96219662744, 2.23138519225,
+        0.0,
+    };
+    double re[9], im[9];
+    USING_FFT(16);
+    if (eps < 1e-11) {
+        eps = 1e-11;
+    }
+    fft.forward(in, re, im);
+    COMPARE_ARR(re, expected_re, 9);
+    COMPARE_ARR(im, expected_im, 9);
+    double back[16];
+    fft.inverse(re, im, back);
+    COMPARE_SCALED(back, in, 16);
+}
+
+/* This one has data from a PRNG, with a fixed seed. Must pass two
+ * tests: (i) same as DFT; (ii) inverse produces original input (after
+ * scaling) */
+ALL_IMPL_AUTO_TEST_CASE(random)
+{
+    const int n = 64;
+    double *in = new double[n];
+    double *re = new double[n/2 + 1];
+    double *im = new double[n/2 + 1];
+    double *re_compare = new double[n/2 + 1];
+    double *im_compare = new double[n/2 + 1];
+    double *back = new double[n];
+    srand48(0);
+    for (int i = 0; i < n; ++i) {
+        in[i] = drand48() * 4.0 - 2.0;
+    }
+    USING_FFT(n);
+    if (eps < 1e-11) {
+        eps = 1e-11;
+    }
+    fft.forward(in, re, im);
+    fft.inverse(re, im, back);
+    FFT::setDefaultImplementation("dft");
+    fft.forward(in, re_compare, im_compare);
+    COMPARE_ARR(re, re_compare, n/2 + 1);
+    COMPARE_ARR(im, im_compare, n/2 + 1);
+    COMPARE_SCALED_N(back, in, n, n);
+    delete[] back;
+    delete[] im_compare;
+    delete[] re_compare;
+    delete[] im;
+    delete[] re;
+    delete[] in;
+}
+
 BOOST_AUTO_TEST_SUITE_END()
