@@ -6,7 +6,7 @@
     A small library for vector arithmetic and allocation in C++ using
     raw C pointer arrays.
 
-    Copyright 2007-2016 Particular Programs Ltd.
+    Copyright 2007-2021 Particular Programs Ltd.
 
     Permission is hereby granted, free of charge, to any person
     obtaining a copy of this software and associated documentation
@@ -50,6 +50,7 @@
 
 #ifdef HAVE_VDSP
 #include <Accelerate/Accelerate.h>
+#include <alloca.h>
 #endif
 
 #include <cstring>
@@ -110,7 +111,7 @@ template<typename T>
 inline void v_zero(T *const BQ_R__ vec,
                    const int count)
 {
-    const T value = T(0);
+    const T value = T();
     for (int i = 0; i < count; ++i) {
         vec[i] = value;
     }
@@ -547,6 +548,30 @@ inline void v_scale(double *const BQ_R__ srcdst,
 }
 #endif // HAVE_IPP
 
+/**
+ * v_clamp
+ *
+ * Clamp the elements in the vector \arg srcdst, of length \arg count,
+ * into the range \arg min to \arg max inclusive.
+ */
+template<typename T, typename G>
+inline void v_clamp(T *const BQ_R__ srcdst,
+                    const G min,
+                    const G max,
+                    const int count)
+{
+    for (int i = 0; i < count; ++i) {
+        if (srcdst[i] < min) {
+            srcdst[i] = min;
+        }
+    }
+    for (int i = 0; i < count; ++i) {
+        if (srcdst[i] > max) {
+            srcdst[i] = max;
+        }
+    }
+}
+
 /**
  * v_increment
  *
@@ -572,9 +597,9 @@ inline void v_increment(T *const BQ_R__ srcdst,
  *
  * Caller guarantees that \arg src and \arg srcdst are non-overlapping.
  */
-template<typename T>
+template<typename T, typename S>
 inline void v_multiply(T *const BQ_R__ srcdst,
-                       const T *const BQ_R__ src,
+                       const S *const BQ_R__ src,
                        const int count)
 {
     for (int i = 0; i < count; ++i) {
@@ -609,10 +634,10 @@ inline void v_multiply(double *const BQ_R__ srcdst,
  * Caller guarantees that \arg src1, \arg src2 and \arg dst are
  * non-overlapping.
  */
-template<typename T>
+template<typename T, typename S>
 inline void v_multiply_to(T *const BQ_R__ dst,
                           const T *const BQ_R__ src1,
-                          const T *const BQ_R__ src2,
+                          const S *const BQ_R__ src2,
                           const int count)
 {
     for (int i = 0; i < count; ++i) {
@@ -735,9 +760,9 @@ inline T v_sum(const T *const BQ_R__ src,
 /**
  * v_multiply_and_sum
  *
- * Multiply the corresponding elements of the vectors \arg src1 and
- * \arg src2, both of length arg \count, sum the results, and return
- * the sum as a scalar value.
+ * Vector dot-product. Multiply the corresponding elements of the
+ * vectors \arg src1 and \arg src2, both of length arg \count, sum the
+ * results, and return the sum as a scalar value.
  *
  * Caller guarantees that \arg src1 and \arg src2 are non-overlapping.
  */
@@ -753,6 +778,46 @@ inline T v_multiply_and_sum(const T *const BQ_R__ src1,
     return result;
 }
 
+#if defined HAVE_IPP
+template<>
+inline float v_multiply_and_sum(const float *const BQ_R__ src1,
+                                const float *const BQ_R__ src2,
+                                const int count)
+{
+    float dp;
+    ippsDotProd_32f(src1, src2, count, &dp);
+    return dp;
+}
+template<>
+inline double v_multiply_and_sum(const double *const BQ_R__ src1,
+                                 const double *const BQ_R__ src2,
+                                 const int count)
+{
+    double dp;
+    ippsDotProd_64f(src1, src2, count, &dp);
+    return dp;
+}
+#elif defined HAVE_VDSP
+template<>
+inline float v_multiply_and_sum(const float *const BQ_R__ src1,
+                                const float *const BQ_R__ src2,
+                                const int count)
+{
+    float dp;
+    vDSP_dotpr(src1, 1, src2, 1, &dp, count);
+    return dp;
+}
+template<>
+inline double v_multiply_and_sum(const double *const BQ_R__ src1,
+                                 const double *const BQ_R__ src2,
+                                 const int count)
+{
+    double dp;
+    vDSP_dotprD(src1, 1, src2, 1, &dp, count);
+    return dp;
+}
+#endif
+
 /**
  * v_log
  *
@@ -790,7 +855,7 @@ template<>
 inline void v_log(float *const BQ_R__ srcdst,
                   const int count)
 {
-    float tmp[count];
+    float *tmp = (float *)alloca(count * sizeof(float));
     vvlogf(tmp, srcdst, &count);
     v_copy(srcdst, tmp, count);
 }
@@ -798,7 +863,7 @@ template<>
 inline void v_log(double *const BQ_R__ srcdst,
                   const int count)
 {
-    double tmp[count];
+    double *tmp = (double *)alloca(count * sizeof(double));
     vvlog(tmp, srcdst, &count);
     v_copy(srcdst, tmp, count);
 }
@@ -841,7 +906,7 @@ template<>
 inline void v_exp(float *const BQ_R__ srcdst,
                   const int count)
 {
-    float tmp[count];
+    float *tmp = (float *)alloca(count * sizeof(float));
     vvexpf(tmp, srcdst, &count);
     v_copy(srcdst, tmp, count);
 }
@@ -849,7 +914,7 @@ template<>
 inline void v_exp(double *const BQ_R__ srcdst,
                   const int count)
 {
-    double tmp[count];
+    double *tmp = (double *)alloca(count * sizeof(double));
     vvexp(tmp, srcdst, &count);
     v_copy(srcdst, tmp, count);
 }
@@ -892,7 +957,7 @@ template<>
 inline void v_sqrt(float *const BQ_R__ srcdst,
                    const int count)
 {
-    float tmp[count];
+    float *tmp = (float *)alloca(count * sizeof(float));
     vvsqrtf(tmp, srcdst, &count);
     v_copy(srcdst, tmp, count);
 }
@@ -900,7 +965,7 @@ template<>
 inline void v_sqrt(double *const BQ_R__ srcdst,
                    const int count)
 {
-    double tmp[count];
+    double *tmp = (double *)alloca(count * sizeof(double));
     vvsqrt(tmp, srcdst, &count);
     v_copy(srcdst, tmp, count);
 }
@@ -969,7 +1034,7 @@ template<>
 inline void v_abs(float *const BQ_R__ srcdst,
                   const int count)
 {
-    float tmp[count];
+    float *tmp = (float *)alloca(count * sizeof(float));
 #if (defined(MACOSX_DEPLOYMENT_TARGET) && MACOSX_DEPLOYMENT_TARGET <= 1070 && MAC_OS_X_VERSION_MIN_REQUIRED <= 1070)
     vvfabf(tmp, srcdst, &count);
 #else
@@ -1104,6 +1169,60 @@ inline void v_fftshift(T *const BQ_R__ vec,
     }
 }
 
+/**
+ * v_min
+ *
+ * Return the minimum of the values in the vector \arg vec, of length
+ * \arg count. For an empty vector, return 0.0.
+ */
+template<typename T>
+inline T v_min(const T *const BQ_R__ vec, const int count)
+{
+    if (count == 0) return T(0);
+    T min = vec[0];
+    for (int i = 1; i < count; ++i) {
+        T v = vec[i];
+        if (v < min) min = v;
+    }
+    return min;
+}
+
+/**
+ * v_max
+ *
+ * Return the maximum of the values in the vector \arg vec, of length
+ * \arg count. For an empty vector, return 0.0.
+ */
+template<typename T>
+inline T v_max(const T *const BQ_R__ vec, const int count)
+{
+    if (count == 0) return T(0);
+    T max = vec[0];
+    for (int i = 1; i < count; ++i) {
+        T v = vec[i];
+        if (v > max) max = v;
+    }
+    return max;
+}
+
+/**
+ * v_rms
+ *
+ * Return the RMS value of the values in the vector \arg vec, of
+ * length \arg count. For an empty vector, return 0.0.
+ */
+template<typename T>
+inline T v_rms(const T *const BQ_R__ vec, const int count)
+{
+    T t = T(0);
+    if (count == 0) return t;
+    for (int i = 0; i < count; ++i) {
+        t += vec[i] * vec[i];
+    }
+    t /= T(count);
+    return sqrt(t);
+}
+
 /**
  * v_mean
  *
