@@ -249,6 +249,7 @@ FeatureExtractionModelTransformer::initialise()
 
     for (int j = 0; in_range_for(m_transforms, j); ++j) {
         createOutputModels(j);
+        setCompletion(j, 0);
     }
 
     m_outputMutex.lock();
@@ -266,7 +267,8 @@ FeatureExtractionModelTransformer::deinitialise()
             << QThread::currentThreadId() << endl;
 
     try {
-        delete m_plugin;
+        m_plugin = {}; // does not necessarily delete, as it's a
+                       // shared_ptr, but in the design case it will
     } catch (const std::exception &e) {
         // A destructor shouldn't throw an exception. But at one point
         // (now fixed) our plugin stub destructor could have
@@ -274,7 +276,6 @@ FeatureExtractionModelTransformer::deinitialise()
         SVCERR << "FeatureExtractionModelTransformer: caught exception while deleting plugin: " << e.what() << endl;
         m_message = e.what();
     }
-    m_plugin = nullptr;
 
     m_descriptors.clear();
 }
@@ -360,6 +361,9 @@ FeatureExtractionModelTransformer::createOutputModels(int n)
 
         // Anything with no value and no duration is an instant
 
+        SVDEBUG << "FeatureExtractionModelTransformer::createOutputModels: "
+                << "creating a SparseOneDimensionalModel" << endl;
+        
         out = std::make_shared<SparseOneDimensionalModel>
             (modelRate, modelResolution, false);
 
@@ -419,7 +423,8 @@ FeatureExtractionModelTransformer::createOutputModels(int n)
             bool flexi = settings.value("use-flexi-note-model", false).toBool();
             settings.endGroup();
 
-            SVCERR << "flexi = " << flexi << endl;
+            SVDEBUG << "FeatureExtractionModelTransformer::createOutputModels: "
+                    << "creating a NoteModel (flexi = " << flexi << ")" << endl;
             
             NoteModel *model;
             if (haveExtents) {
@@ -436,6 +441,9 @@ FeatureExtractionModelTransformer::createOutputModels(int n)
 
         } else {
 
+            SVDEBUG << "FeatureExtractionModelTransformer::createOutputModels: "
+                    << "creating a RegionModel" << endl;
+
             RegionModel *model;
             if (haveExtents) {
                 model = new RegionModel
@@ -481,6 +489,11 @@ FeatureExtractionModelTransformer::createOutputModels(int n)
             m_needAdditionalModels[n] = true;
         }
 
+        SVDEBUG << "FeatureExtractionModelTransformer::createOutputModels: "
+                << "creating a SparseTimeValueModel "
+                << "(additional models to come? -> "
+                << m_needAdditionalModels[n] << ")" << endl;
+
         SparseTimeValueModel *model;
         if (haveExtents) {
             model = new SparseTimeValueModel
@@ -504,6 +517,10 @@ FeatureExtractionModelTransformer::createOutputModels(int n)
         // has a fixed sample rate and more than one value per result
         // must be a dense 3D model.
 
+        SVDEBUG << "FeatureExtractionModelTransformer::createOutputModels: "
+                << "creating a BasicCompressedDenseThreeDimensionalModel"
+                << endl;
+        
         auto model =
             new BasicCompressedDenseThreeDimensionalModel
             (modelRate, modelResolution, binCount, false);
@@ -1156,7 +1173,9 @@ FeatureExtractionModelTransformer::addFeature(int n,
             <BasicCompressedDenseThreeDimensionalModel>(outputId);
         if (!model) return;
         
-        DenseThreeDimensionalModel::Column values = feature.values;
+        DenseThreeDimensionalModel::Column values;
+        values.insert(values.begin(),
+                      feature.values.begin(), feature.values.end());
         
         if (!feature.hasTimestamp && m_fixedRateFeatureNos[n] >= 0) {
             model->setColumn(m_fixedRateFeatureNos[n], values);
